import requests
import json
import re
from datetime import datetime

BASE_URL = 'https://nhentai.sinxdr.workers.dev'
BASE_IMG = 'https://i.nhentai.net/galleries'

def auto_space(string):
    """Automatically adds spaces for GET requests

    Parameters
    ----------
    string : str
        The string to be formatted.

    Returns
    -------
    str

    """
    return string.replace(' ', '+')


def just_number(string):
    """Remove the non-numeric characters from a string.

    Parameters
    ----------
    string : str
        The desired string.

    Returns
    -------
    str
    """
    return re.sub(r'\D', '', string)

def better_object(parser):
    """Converts the json object to a more readable object.

    Parameters
    ----------
    parser : dict
        
    Returns
    -------
    dict

    """
    return json.dumps(parser, sort_keys=True, indent=4, ensure_ascii=False)


def readable_timestamp(timestamp):
    """Converts a timestamp to a datetime object.

    Parameters
    ----------
    timestamp : int
        The timestamp to be converted.

    Returns
    -------
    str
    """
    return datetime.utcfromtimestamp(int(timestamp)).strftime('%Y-%m-%d %H:%M:%S')

def parser(obj: dict):
        raw_object = json.loads(better_object(obj), encoding="utf-8")
        media = raw_object['media_id']

        book_object = {
            'details': {
                'id': raw_object['id'],
                'title': raw_object['title'],
                'link': f'https://nhentai.net/g/{raw_object["id"]}',
                'upload_date': readable_timestamp(raw_object['upload_date'])
            },
            'scanlator': raw_object['scanlator'],
            'num_pages': raw_object['num_pages'],
            'num_favorites': raw_object['num_favorites']}

        tags = []
        for tag in raw_object['tags']:
            tags.append(tag['name'])
        book_object['tags'] = tags

        for tag in raw_object['tags']:
            if tag['type'] == 'language':
                book_object['language'] = tag['name']
                break

        image_urls = []
        for image in raw_object['images']['pages']:

            if image['t'] == 'p':
                image_urls.append(
                    f'{BASE_IMG}/{media}/{len(image_urls) + 1}.png')

            elif image['t'] == 'j':
                image_urls.append(
                    f'{BASE_IMG}/{media}/{len(image_urls) + 1}.jpg')

            elif image['t'] == 'g':
                image_urls.append(
                    f'{BASE_IMG}/{media}/{len(image_urls) + 1}.gif')

        book_object['image_urls'] = image_urls

        thumbnail_urls = []
        for image in raw_object['images']['pages']:
            if image['t'] == 'p':
                thumbnail_urls.append(
                    f'{BASE_IMG}/{media}/{len(thumbnail_urls) + 1}t.png')

            elif image['t'] == 'j':
                thumbnail_urls.append(
                    f'{BASE_IMG}/{media}/{len(thumbnail_urls) + 1}t.jpg')

            elif image['t'] == 'g':
                thumbnail_urls.append(
                    f'{BASE_IMG}/{media}/{len(thumbnail_urls) + 1}t.gif')

        book_object['thumbnail_urls'] = thumbnail_urls
        return better_object(book_object)


import requests
import json
import re
from datetime import datetime
from janda.utils import parser, just_number

BASE_URL = 'https://nhentai.sinxdr.workers.dev'
BASE_IMG = 'https://i.nhentai.net/galleries'

class Nhentai(object):

    @staticmethod
    def preg_match_tags(tags):
        """Parses the tags and returns a list of tags.

        Parameters
        ----------
        tags : str
            The tags to be parsed.
        
        Returns
        -------
        list
        """

        tags_name = []
        for tag in tags:
            tags_name.append(tag['name'])
        return tags_name

    @staticmethod
    def get_language_in_tags(tags):
        """Parses language value

        Parameters
        ----------
        tags : list
            The tags to be parsed.

        Returns
        -------
        str
        """

        for tag in tags:
            if tag['type'] == 'language':
                return tag['name']
    


    def __init__(self, api_key: str = ''):
        """Initializes the client.

        Parameters
        ----------
        api_key : str
            scathach.dev API key (optional)
        
        Returns
        -------
        None
        """
        if api_key is '':
            self.api_key = None
        else:
            self.api_key = api_key
        self.specs = {'api_key': self.api_key}

    async def get_book(self, book: int, safe_search: bool = None):
        """Get doujin API from Id

        Parameters
        ----------
        book : int
            Nmber id of the book

        safe_search : bool
            If True, janda will throw you error whenever contains minor content, such as loli or shota. Default is False

        Raises
        ------
        ValueError
            If the doujin is not found.

        Returns
        -------
        dict
        """

        try:
            if isinstance(book, int):
                book = str(book)

            self.book = str(just_number(book))

        except ValueError:
            raise ValueError('Book must be an int')

        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}')
        if data.status_code == 404:
            raise ValueError('Book not found')

        if data.status_code != 200:
            raise ValueError('Request failed')

        return parser(data.json())

    async def search(self, tags: str, page: int = 1, popular: str = 'today'):
        """Search doujin by tags / artis / character / parody or group

        Parameters
        ----------
        tags : str
            Tags to search for.

        page : int
            Page number. Default is 1.

        popular : str
            Popularity type. Default is today.

        Raises
        ------
        ValueError
            If the doujin is not found.

        Returns
        -------
        dict
        """

        if popular not in ['today', 'all', 'week']:
            raise ValueError('popular must be today, all, or week')

        self.specs['query'] = tags
        self.specs['page'] = page
        self.specs['popular'] = popular

        data = requests.get(
            f'{BASE_URL}/api/galleries/search', params=self.specs)

        if data.status_code != 200:
            raise ValueError('Request failed')

        if data.status_code != 200:
            raise ValueError('Request failed')

        self.raw_object = json.loads(Nhentai.better_object(data.json()))
        self.results = self.raw_object['result']

        self.results_object = []
        for result in self.results:
            self.results_object.append({
                'id': result['id'],
                'title': result['title'],
                'link': f'https://nhentai.net/g/{result["id"]}',
                'upload_date': Nhentai.readable_timestamp(result['upload_date']),
                'num_pages': result['num_pages'],
                'num_favorites': result['num_favorites'],
                'language': Nhentai.get_language_in_tags(result['tags']),
                'tags': Nhentai.preg_match_tags(result['tags'])
            })

        return Nhentai.better_object(self.results_object)

    async def get_related(self, book: int):
        """Get realated book API from book ID or book link

        Parameters
        ----------
        book : int
            Nmber id of the book

        Raises
        ------
        ValueError
            If the doujin is not found.

        Returns
        -------
        dict
        """

        try:
            self.book = str(Nhentai.just_number(book))

        except ValueError:
            raise ValueError('Book must be an int')

        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}/related')

        if data.status_code != 200:
            raise ValueError('Request failed')

        if data.status_code != 200:
            raise ValueError('Request failed')

        self.raw_object = json.loads(Nhentai.better_object(data.json()))
        self.results = self.raw_object['result']

        self.results_object = []
        for result in self.results:
            self.results_object.append({
                'id': result['id'],
                'title': result['title'],
                'link': f'https://nhentai.net/g/{result["id"]}',
                'upload_date': Nhentai.readable_timestamp(result['upload_date']),
                'num_pages': result['num_pages'],
                'num_favorites': result['num_favorites'],
                'language': Nhentai.get_language_in_tags(result['tags']),
                'tags': Nhentai.preg_match_tags(result['tags'])
            })

        return Nhentai.better_object(self.results_object)

    async def random(self):
        """Get random doujin
        
        Raises
        ------
        ValueError
            If the doujin is not found.

        Returns
        -------
        dict
        """
        random = requests.get(F'{BASE_URL}/random')

        try:
            self.book = str(Nhentai.just_number(random.url))

        except ValueError:
            raise ValueError('Uh oh something wrong here')

        data = requests.get(F'{BASE_URL}/api/gallery/{self.book}')
        if data.status_code == 404:
            raise ValueError('Error')

        if data.status_code != 200:
            raise ValueError('Request failed')

        return Nhentai.parser(data.json())



import re

from setuptools import setup


version = ''
with open('janda/__init__.py') as f:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)


requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()


if not version:
    raise RuntimeError('version is not set')

readme = ''
with open('README.md') as f:
    readme = f.read()


setup(
    name='janda',
    author='sinkaroid',
    author_email='anakmancasan@gmail.com',
    version='1.0.10',
    long_description=readme,
    url='https://github.com/sinkaroid/janda',
    packages=['janda'],
    license='MIT',
    classifiers=[
        "Framework :: AsyncIO",
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        "Natural Language :: English",
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 3.7',
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",

    ],
    description='Doujin wrapper',
    include_package_data=True,
    keywords=['janda', 'doujin', 'wrapper'],
    install_requires=requirements
)
